-- Most of the addon code and fixes are by Vrul (really much appreciated)
-- You can check the forum here:
-- http://www.wowinterface.com/forums/showthread.php?t=54975
-- 

local ADDON, namespace = ...
local L = namespace.L

local addon = {}		
local addonsort = {}

local UPDATEPERIOD = 5
local elapsed = 0

local fontName, fontHeight, fontFlags = GameFontNormal:GetFont()
local MaxSlidersAddons

local mediapath = "interface\\addons\\"..ADDON.."\\media\\"

-- I have most of the UI hidden and some of these entries are nil.
-- So I cant use them and I have to rely on custom strings to be localized
-- Here they remain as remainder only because they are not used in the code. 
local EntryText = {
	_G.CharacterMicroButton.tooltipText,  	-- nil for my UI
	_G.SpellbookMicroButton.tooltipText,  	-- nil for my UI
	_G.TalentMicroButton.tooltipText, 		
	_G.AchievementMicroButton.tooltipText,
	_G.QuestLogMicroButton.tooltipText,
	_G.GuildMicroButton.tooltipText,
	_G.LFDMicroButton.tooltipText,
	_G.CollectionsMicroButton.tooltipText,	-- nil for my UI
	_G.EJMicroButton.tooltipText,
}

local GMMENU_ENTRIES = {
	{L["Character"], "ToggleCharacter('PaperDollFrame')","green", "TOGGLECHARACTER0"},
	{L["Spellbook and Abilities"], "ToggleSpellBook(BOOKTYPE_SPELL)", "spells", "TOGGLESPELLBOOK"},
	{L["Specialization and Talents"], "ToggleTalentFrame()", "talents","TOGGLETALENTS"},
	{L["Achievements"], "ToggleAchievementFrame()", "achievements","TOGGLEACHIEVEMENT"},
	{L["Quest Log"], "ToggleQuestLog()", "quest","TOGGLEQUESTLOG"},
	{L["Guild"], "ToggleGuildFrame()", "social","TOGGLEGUILDTAB"}, 
	{L["Group Finder"], "PVEFrame_ToggleFrame()", "lfg","TOGGLEGROUPFINDER"}, 
	{L["Collections"], "ToggleCollectionsJournal(1)", "mounts","TOGGLECOLLECTIONS"}, 
	{L["Adventure Guide"], "ToggleEncounterJournal()", "journal","TOGGLEENCOUNTERJOURNAL"},
	{""},
	{L["Game Menu"], "GameMenuFrame:Show()","", "TOGGLEGAMEMENU"},
	{L["Reload UI"], "ReloadUI()"},
}

local BUTTON_HEIGHT = fontHeight + 4
local BUTTON_SPACING = 0
local MENU_BUFFER = #GMMENU_ENTRIES
local MENU_SPACING = 1
local MENU_WIDTH = 250
local MENU_ROWS = MENU_BUFFER + 20 + 4 -- #GMMENU_ENTRIES + max MAXADDS + empties filling lines

-- default MAXADDS to display
GMMENU = {
	MAXADDS = 1,
	SHOWICON = true,
	SHOWKEYS = false,
}

local numrows = {}
for i = 1, MENU_ROWS do
	numrows[i] = {}
end

local checkbox = {}
for i = 1, #GMMENU do
	checkbox[i] = {}
end
--- Some functions ----------------------------------------------------------------------
local function color(number,text)
	
	if number == nil or number == 0 then return "-" end

	if text == "latency" then
		if number <= 75 then 
			return string.format("|cff00FF00"..number.."|r")
		elseif number > 75 and number < 120 then 
			return string.format("|cffFFFF00"..number.."|r")
		elseif number >= 120 then 
			return string.format("|cffFF0000"..number.."|r")
		end
	elseif text == "fps" then
		if number <= 30 then 
			return string.format("|cffFF0000"..number.."|r")
		elseif number > 30 and number < 59 then 
			return string.format("|cffFFFF00"..number.."|r")
		elseif number >= 59 then 
			return string.format("|cff00FF00"..number.."|r")
		end
	elseif text == "memory" then 

		if number >= 3 then 
			return string.format("|cffFF0000"..string.format("%.2f mb", number).."|r")
		elseif number > 0.5 and number < 3 then 
			return string.format("|cffFFFF00"..string.format("%.2f mb", number).."|r")
		elseif number <= 0.5 then 
			return string.format("|cff00FF00"..string.format("%.2f mb", number).."|r")
		end
	end
end

local function numaddons()
	local count = 0
	local memTot = 0
	
	addon = {}
	addonsort = {}
	
	UpdateAddOnMemoryUsage()
	for i = 1, GetNumAddOns() do
		local name, _, _, enabled = GetAddOnInfo(i)		
		if IsAddOnLoaded(i) then
			-- ugly hack to sort them later
			addon[GetAddOnMemoryUsage(i)] = name
			table.insert(addonsort, GetAddOnMemoryUsage(i))
			memTot = memTot + (GetAddOnMemoryUsage(i)/1024)
			count = count + 1
		end
		table.sort(addonsort, function(a,b) return a>b end)
	end
	return count , color(memTot,"memory")
end

local function UpdateMenu()
	for i=1, 21 do 
		numrows[MENU_BUFFER+3+i]["l"]:SetText("")
		numrows[MENU_BUFFER+3+i]["r"]:SetText("")
	end
	numrows[MENU_BUFFER+2]["l"]:SetText("-- sync --")
	numrows[MENU_BUFFER+2]["r"]:SetText("-- sync --")
end

--- Create Base Menu Frame ------------------------------------------------------------------------
local Menu = CreateFrame('Frame', nil, UIParent)
Menu:RegisterEvent("ADDON_LOADED")
Menu:RegisterEvent("PLAYER_ENTERING_WORLD")
Menu:SetFrameStrata('TOOLTIP')
Menu:SetClampedToScreen(true)
Menu:SetBackdrop({
	bgFile = "Interface/DialogFrame/UI-DialogBox-Background",
	edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
	tile = true, tileSize = 32, edgeSize = 16,
	insets = { left = 3, right = 3, top = 3, bottom = 3 }
})
Menu:Hide()

local function Menu_OnLeave()
	local focus = GetMouseFocus() or WorldFrame
	if focus ~= Menu and focus:GetParent() ~= Menu then
		Menu:Hide()
	end
end
Menu:SetScript('OnLeave', Menu_OnLeave)

-- top row menu
numrows[1]["l"] = Menu:CreateFontString()
numrows[1]["l"]:SetPoint("TOPLEFT", Menu, "TOPLEFT", MENU_BUFFER, -MENU_BUFFER)
numrows[1]["l"]:SetFont(fontName, fontHeight)
numrows[1]["l"]:SetTextColor(1, 0.8, 0, 1)
numrows[1]["l"]:SetText("gmMenu")

numrows[1]["r"] = Menu:CreateFontString()
numrows[1]["r"]:SetPoint("TOPRIGHT", Menu, "TOPRIGHT", -MENU_BUFFER, -MENU_BUFFER)
numrows[1]["r"]:SetFont(fontName, fontHeight)
numrows[1]["r"]:SetTextColor(1, 1, 1, 1)

--
-- menu builder based on GMMENU_ENTRIES
--

-- anchor1 - blank line
numrows[MENU_BUFFER+1]["l"] = Menu:CreateFontString()
numrows[MENU_BUFFER+1]["l"]:SetPoint("TOPLEFT", Menu, "TOPLEFT", 0, -(MENU_BUFFER+10) * MENU_BUFFER)
numrows[MENU_BUFFER+1]["l"]:SetFont(fontName, fontHeight)
numrows[MENU_BUFFER+1]["l"]:SetTextColor(1, 1, 1, 1)
numrows[MENU_BUFFER+1]["l"]:SetText("")

numrows[MENU_BUFFER+1]["r"] = Menu:CreateFontString()
numrows[MENU_BUFFER+1]["r"]:SetPoint("TOPRIGHT", Menu, "TOPRIGHT", 0, -(MENU_BUFFER+10) * MENU_BUFFER)
numrows[MENU_BUFFER+1]["r"]:SetFont(fontName, fontHeight)
numrows[MENU_BUFFER+1]["r"]:SetTextColor(1, 1, 1, 1)
numrows[MENU_BUFFER+1]["r"]:SetText("")

-- summary addons
numrows[MENU_BUFFER+2]["l"] = Menu:CreateFontString()
numrows[MENU_BUFFER+2]["l"]:SetPoint("BOTTOMLEFT", numrows[MENU_BUFFER+1]["l"], "BOTTOMLEFT", MENU_BUFFER, -MENU_BUFFER)
numrows[MENU_BUFFER+2]["l"]:SetFont(fontName, fontHeight)
numrows[MENU_BUFFER+2]["l"]:SetTextColor(1, 1, 1, 1)
numrows[MENU_BUFFER+2]["l"]:SetText("-- sync --")

numrows[MENU_BUFFER+2]["r"] = Menu:CreateFontString()
numrows[MENU_BUFFER+2]["r"]:SetPoint("BOTTOMRIGHT", numrows[MENU_BUFFER+1]["r"], "BOTTOMRIGHT", -MENU_BUFFER, -MENU_BUFFER)
numrows[MENU_BUFFER+2]["r"]:SetFont(fontName, fontHeight)
numrows[MENU_BUFFER+2]["r"]:SetTextColor(1, 1, 1, 1)
numrows[MENU_BUFFER+2]["r"]:SetText("-- sync --")

-- anchor11 - blank line
numrows[MENU_BUFFER+3]["l"] = Menu:CreateFontString()
numrows[MENU_BUFFER+3]["l"]:SetPoint("TOPLEFT", numrows[MENU_BUFFER+2]["l"], "TOPLEFT", 0, -MENU_BUFFER)
numrows[MENU_BUFFER+3]["l"]:SetFont(fontName, fontHeight)
numrows[MENU_BUFFER+3]["l"]:SetTextColor(1, 1, 1, 1)
numrows[MENU_BUFFER+3]["l"]:SetText("")

numrows[MENU_BUFFER+3]["r"] = Menu:CreateFontString()
numrows[MENU_BUFFER+3]["r"]:SetPoint("TOPRIGHT", numrows[MENU_BUFFER+2]["r"], "TOPRIGHT", 0, -MENU_BUFFER)
numrows[MENU_BUFFER+3]["r"]:SetFont(fontName, fontHeight)
numrows[MENU_BUFFER+3]["r"]:SetTextColor(1, 1, 1, 1)
numrows[MENU_BUFFER+3]["r"]:SetText("")

-- cycle top hungry addons 
-- create max GMMENU["MAXADDS"] object numrows (20)

for i=1, 21 do 
	numrows[MENU_BUFFER+3+i]["l"] = Menu:CreateFontString()
	numrows[MENU_BUFFER+3+i]["l"]:SetPoint("TOPLEFT", numrows[MENU_BUFFER+2+i]["l"], "TOPLEFT", 0, -MENU_BUFFER-5)
	numrows[MENU_BUFFER+3+i]["l"]:SetFont(fontName, fontHeight)
	numrows[MENU_BUFFER+3+i]["l"]:SetTextColor(1, 1, 1, 1)
	numrows[MENU_BUFFER+3+i]["l"]:SetText("")

	numrows[MENU_BUFFER+3+i]["r"] = Menu:CreateFontString()
	numrows[MENU_BUFFER+3+i]["r"]:SetPoint("TOPRIGHT", numrows[MENU_BUFFER+2+i]["r"], "TOPRIGHT", 0, -MENU_BUFFER-5)
	numrows[MENU_BUFFER+3+i]["r"]:SetFont(fontName, fontHeight)
	numrows[MENU_BUFFER+3+i]["r"]:SetTextColor(1, 1, 1, 1)
	numrows[MENU_BUFFER+3+i]["r"]:SetText("")
end


--- Create LDB top --------------------------------------------------------------------------------
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local dataobj = ldb:NewDataObject("gmMenu", {
	type = "data source",
	icon = mediapath.."icon.tga",
	text = "-- sync --",
	
	OnEnter = function(self)
		if not Menu:IsShown() then
			local _, selfCenter = self:GetCenter()
			local _, uiCenter = UIParent:GetCenter()
			Menu:ClearAllPoints()
			if selfCenter >= uiCenter then
				Menu:SetPoint('TOP', self, 'BOTTOM')
			else
				Menu:SetPoint('BOTTOM', self, 'TOP')
			end
			Menu:Show()
		end
	end,
	OnLeave = Menu_OnLeave	
})

local function UpdateLDB()
end

function dataobj.OnClick(self, button)    

	if button == "LeftButton" then
		UpdateLDB()
	end
	
	if button == "RightButton" then
		InterfaceOptionsFrame_Show(options)
	end
	Menu:Hide()
end

local function Button_OnClick(self)
	Menu:Hide()
	assert(loadstring(GMMENU_ENTRIES[self.index][2]))()
end

-- Configuration Panel ----------------------------------------------------------------------------

local options = CreateFrame("Frame", "options", InterfaceOptionsFramePanelContainer)
options.name = GetAddOnMetadata(ADDON, "Title") or ADDON
InterfaceOptions_AddCategory(options)

-- Title (title)
local title = options:CreateFontString("$parentTitle", "ARTWORK", "GameFontNormalLarge")
title:SetPoint("TOPLEFT", 16, -16)
title:SetText(options.name)
options.title = title

checkbox[1] = CreateFrame("CheckButton", "$parentEnable", options, "InterfaceOptionsCheckButtonTemplate")
checkbox[1]:SetPoint("TOPLEFT", 18, -100)
checkbox[1].Text:SetText(L["Show Icons"])
checkbox[1]:SetChecked(GMMENU["SHOWICON"])
checkbox[1]:SetScript("OnClick", function(self)
	if checkbox[1]:GetChecked() then
		GMMENU["SHOWICON"] = true
	else 
		GMMENU["SHOWICON"] = false
	end	
end)

checkbox[2] = CreateFrame("CheckButton", "$parentEnable", options, "InterfaceOptionsCheckButtonTemplate")
checkbox[2]:SetPoint("TOPLEFT", 18, -130)
checkbox[2].Text:SetText(L["Show Bind Keys"])
checkbox[2]:SetChecked(GMMENU["SHOWICON"])
checkbox[2]:SetScript("OnClick", function(self)
	if checkbox[2]:GetChecked() then
		GMMENU["SHOWKEYS"] = true
	else 
		GMMENU["SHOWKEYS"] = false
	end	
end)

-- Desc Text
local phrasedesc = options:CreateFontString(nil, nil, "GameFontHighlight")
phrasedesc:SetPoint("TOPLEFT", 18, -180)
phrasedesc:SetText(L["Number of addons:"])	

local slider1 = CreateFrame("Slider", "", options, "OptionsSliderTemplate")
-- slider1:SetPoint("BOTTOMLEFT", 10, 150)
slider1:SetPoint("TOPLEFT", 18, -200)
slider1:SetWidth(200)
-- slider1:SetMinMaxValues(1, MaxSlidersAddons)
slider1:SetValueStep(1)
slider1.value = slider1:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
slider1.value:SetPoint("TOP", slider1, "BOTTOM", 0, 0)
slider1:SetScript("OnValueChanged", function(self, value)
	self:SetValue(floor(value + 0.5))
	self.value:SetText(floor(value + 0.5))
	GMMENU["MAXADDS"] = floor(value + 0.5)
	Menu:SetSize(MENU_WIDTH, BUTTON_HEIGHT + (5 + MENU_BUFFER + GMMENU["MAXADDS"]) * (BUTTON_HEIGHT + BUTTON_SPACING + MENU_SPACING))
	UpdateMenu()
end)

local button1 = CreateFrame("button", button1, options, "UIPanelButtonTemplate")
button1:SetHeight(40)
button1:SetWidth(150)
button1:SetPoint("BOTTOMLEFT", 10, 16)
button1:SetText(L["Reset and reload UI"])
button1.tooltipText = L["Reset settings and reload UI"]
button1:SetScript("OnClick",  
	function()
		GMMENU = {
			MAXADDS = 1,
			SHOWICON = true,
			SHOWKEYS = false,
		}
		ReloadUI()
	end)  

local button2 = CreateFrame("button", button2, options, "UIPanelButtonTemplate")
button2:SetHeight(40)
button2:SetWidth(150)
button2:SetPoint("BOTTOMLEFT", 190, 16)
button2:SetText(L["Save and reload UI"])
button2.tooltipText = L["Save settings and reload your UI"]
button2:SetScript("OnClick",  
	function()
		ReloadUI()
	end)  

--- credits  
local credits = options:CreateFontString("$parentTitle", "ARTWORK", "SystemFont_Small")
credits:SetPoint("BOTTOMRIGHT", -16, 16)
credits:SetText(L["version"].." |cffffd200"..GetAddOnMetadata(ADDON, "Version").."|r by |cffffd200"..GetAddOnMetadata(ADDON, "Author").."|r")
options.credits = credits		

function options.refresh()
	slider1:SetValue(GMMENU["MAXADDS"])
	checkbox[1] = SetChecked(GMMENU["SHOWICON"])
	checkbox[2] = SetChecked(GMMENU["SHOWKEYS"])
end

if LibStub and LibStub("LibAboutPanel", true) then
	options.about = LibStub("LibAboutPanel").new(options.name, ADDON)
end

--- Startup Builders ------------------------------------------------------------------------------
Menu:SetScript('OnEvent', function(self, event)
	Menu:UnregisterEvent(event)

	if event == "ADDON_LOADED" and arg1 == "gmMenu" then 
	end

	if event == "PLAYER_ENTERING_WORLD" then 
				
		for index = 1, #GMMENU_ENTRIES do
		
			if GMMENU_ENTRIES[index] and GMMENU_ENTRIES[index] ~= "" then

				local button = CreateFrame("Button", nil, Menu)
				
				if index ~= 1 then
					button:SetPoint("TOPLEFT", Menu[index - 1], "BOTTOMLEFT", 0, -BUTTON_SPACING)
				else
					button:SetPoint("TOPLEFT", numrows[1]["l"], "BOTTOMLEFT", 0, -MENU_BUFFER)
				end
				
				button:SetPoint("RIGHT", -MENU_BUFFER, 0)
				button:SetHeight(BUTTON_HEIGHT)
				button:SetNormalFontObject("GameFontNormal")
				button:SetHighlightTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
				button.index = index
				button:SetScript("OnClick", Button_OnClick)
				button:SetScript("OnLeave", Menu_OnLeave)
				Menu[index] = button

				local text = button:CreateFontString(ADDON .. "btn_font", nil, "GameFontNormal")
				text:SetAllPoints()
				text:SetJustifyH("LEFT")
				text:SetJustifyV("MIDDLE")
				text:SetTextColor(1, 1, 1, 1)
				button:SetFontString(text)
				
				if GMMENU_ENTRIES[index][3] and GMMENU_ENTRIES[index][3] ~="" then	
					if GMMENU["SHOWICON"] and GMMENU["SHOWKEYS"] then 
						button:SetFormattedText("|T%s:0|t %s |cffFFFF00(%s)|r",mediapath..GMMENU_ENTRIES[index][3], GMMENU_ENTRIES[index][1], GetBindingKey(GMMENU_ENTRIES[index][4]))
					elseif GMMENU["SHOWICON"] == false and GMMENU["SHOWKEYS"] then
						button:SetFormattedText("%s |cffFFFF00(%s)|r)", GMMENU_ENTRIES[index][1], GetBindingKey(GMMENU_ENTRIES[index][4]))
					elseif 	GMMENU["SHOWICON"] and GMMENU["SHOWKEYS"] == false then
						button:SetFormattedText("|T%s:0|t %s",mediapath..GMMENU_ENTRIES[index][3], GMMENU_ENTRIES[index][1])
					elseif GMMENU["SHOWICON"] == false and GMMENU["SHOWKEYS"] == false then
						button:SetFormattedText(GMMENU_ENTRIES[index][1])
					end
				else
					button:SetFormattedText(GMMENU_ENTRIES[index][1])
				end
			
			else
				
				Menu[index] = Menu[index - 1]
			
			end		
		end			
			
		if numaddons() < GMMENU["MAXADDS"] then 
			GMMENU["MAXADDS"] = numaddons()
		end 
		
		Menu:SetSize(MENU_WIDTH, BUTTON_HEIGHT + (5 + MENU_BUFFER + GMMENU["MAXADDS"]) * (BUTTON_HEIGHT + BUTTON_SPACING + MENU_SPACING))
		UpdateMenu()
		UpdateLDB()
	
		if numaddons() < 20 then 
			MaxSlidersAddons = numaddons()
		else
			MaxSlidersAddons = 20
		end
	
		if GMMENU["MAXADDS"] <= MaxSlidersAddons then  
			slider1:SetValue(GMMENU["MAXADDS"])
		else 
			slider1:SetValue(MaxSlidersAddons)
		end		
		slider1:SetMinMaxValues(1, MaxSlidersAddons)
		checkbox[1]:SetChecked(GMMENU["SHOWICON"])
		checkbox[2]:SetChecked(GMMENU["SHOWKEYS"])
	end
end)

local framestats = CreateFrame("Frame")
framestats:SetScript("OnUpdate", function(self, elap)
	elapsed = elapsed + elap
	if elapsed < UPDATEPERIOD then 
		return 
	else
		elapsed = 0
		dataobj.text = "H:"..color(select(3,GetNetStats()) or 0,"latency").." W:"..color(select(4,GetNetStats()) or 0,"latency").." F:"..color(floor(GetFramerate()) or 0,"fps")

		numrows[MENU_BUFFER+2]["l"]:SetText(L["Total addon(s):"] .. " " .. format("|cff00ff00%d|r", select(1,numaddons())))	
		numrows[MENU_BUFFER+2]["r"]:SetText(select(2,numaddons()))
		
		for i = 1, GMMENU["MAXADDS"] do
			numrows[MENU_BUFFER+3+i]["l"]:SetText(addon[addonsort[i]])
			numrows[MENU_BUFFER+3+i]["r"]:SetText(color(addonsort[i]/1024, "memory"))
		end
	end
end)